{ *************************************************
  Here is a simple routine to allow you to use TEGL
  proportional fonts in plain old BGI graphics.

  Why would you want to?  Well, in my case I use the
  Ryle printer drivers concurrently with the TEGL graphics
  drivers.  I can send my stuff to the BGI graphics printer
  driver (proportional TEGL fonts!) at the same time I
  am updating my screen with a status bar (or whatever).

  So, for whatever reason, you can use these routines to
  use TEGL fonts, proportionally spaced, in your own BGI code.

  *** December 16, 1994
  *** Jay Faubion
  *** CI$ 72763,200
  *** jfaubion@iohk.com  (Hong Kong)

  MANY THANKS to Richard Tom for a really nice package.
  }



program TEGLBGI;

uses
  graph, teglfont;

type TFontArray = array[1..265] of Byte;
  { 10 byte header, then 255 charwidth table}

const
  farray    : array[1..29] of Pointer =
  (@pc3270, @broadway, @gaelic, @lcdfont, @lcdfont,
   @litalic, @sansx19, @wndwx19, @font14, @font09, @pc24,
   @apls7, @countdwn, @georgian, @light14, @m3270, @script2,
   @ega09, @font07, @future, @pc9, @brdwx19, @fraktur,
   @italic, @light9, @ocr, @script, @font07cp, @oenglish);

var
  i, gd, gm : Integer;
  astr      : String;

  {  The TEGL font header looks like this...
    fontptr	    = ^fontinfo;
    fontinfo	    = Record
	    Fontsignature	 : word;
	    FontHeight	   : byte;
	    FontWidth	     : byte;
	    FontAsciiStart : byte;
	    FontAsciiEnd	 : byte;
	    FontTop 	     : byte;
	    FontBase	     : byte;
	    FontBottom	   : byte;
	    FontByteWidth	 : byte;
    end;
    .. followed by 255 bytes of proportional character widths,
    .. followed by the bitmap array
       (FontAsciiEnd - FontAsciiStart) characters of
          [1..FontByteWidth, 1..FontHeight];
}
  procedure GOutCharXY(var x, y : Integer; c : Char; fnt : Pointer);
  var Color, i, xx : Integer;
    f         : ^TFontArray;
    Cstart, CRange, BHeight, BWidth : Byte;
  begin
    Color := 1;
    f := fnt;
    xx := x;                      { Save it so we can reset it each new pixel line}
    Cstart := f^[5];              { TEGLCharStart    }
    CRange := f^[6];              { TEGLCharRange    }
    BWidth := f^[10];             { font byte width  }
    BHeight := f^[3];             { font byte height }
    if not(c in [Char(Cstart) ..Char(CRange)]) then Exit;
    { if char not in font then exit! }
    {$R-}                         { range checking off for a sec.. }
    f := Addr(f^[266+(Byte(c)-Cstart)*BWidth*BHeight]);
    {$R+}                         { skip to char bitmap }
    Dec(y, BHeight);
    repeat
      xx := x;                    { start each line at x }
      for i := 1 to BWidth do begin
        if (f^[i] and 1) = 1 then PutPixel(xx+7, y, Color);
        if (f^[i] and 2) = 2 then PutPixel(xx+6, y, Color);
        if (f^[i] and 4) = 4 then PutPixel(xx+5, y, Color);
        if (f^[i] and 8) = 8 then PutPixel(xx+4, y, Color);
        if (f^[i] and 16) = 16 then PutPixel(xx+3, y, Color);
        if (f^[i] and 32) = 32 then PutPixel(xx+2, y, Color);
        if (f^[i] and 64) = 64 then PutPixel(xx+1, y, Color);
        if (f^[i] and 128) = 128 then PutPixel(xx+0, y, Color);
        if i <> BWidth then Inc(xx, 8); {increment to full line width}
      end;
      Dec(BHeight);
      f := Addr(f^[i+1]);
      Inc(y);
    until BHeight = 0;
    f := fnt;
    Inc(x, f^[Byte(c)+10]);       {TEGL font proportional width }
  end;


  procedure GOutTextXY(x, y : Integer; AscStr : String; font : Pointer);
  var
    i         : Integer;
  begin
    for i := 1 to Length(AscStr) do
      GOutCharXY(x, y, AscStr[i], font);
  end;

begin
  astr := 'Proportional TEGL fonts in BGI';
  gd := Detect;
  InitGraph(gd, gm, 'F:\bp\bgi');
  if grOk <> 0 then Halt;
  setcolor(15);
  bar(0, 0, getmaxx, getmaxy);
  setcolor(0);
  for i := 1 to 15 do
    GOutTextXY(10, i*30, astr, farray[i]);
  for i := 16 to 29 do
    GOutTextXY(getmaxx div 2, (i-15)*30, astr, farray[i]);
  ReadLn;
  restorecrtmode;
end.
